import os
import sys
import pandas
from PyQt5.QtWidgets import (QMainWindow, QApplication, QGroupBox, QHBoxLayout,
                             QLabel, QPushButton, QVBoxLayout, QWidget, QTabWidget,
                             QComboBox, QCheckBox, QListWidget, QAction, QFileDialog)
from PyQt5.QtGui import QIcon

sys.path.append('../')

import settings
from script.deputat import (AllTeachers, AllClasses, SUBJECT_SHORT_DICT,
                            pretty_out_classes, pretty_out_teachers)
from GUI.popups import AddTeacherPopUp, AddClassPopUp, QuitPopUp


class MainWindow(QMainWindow):
    icon_path = settings.icon_dir()
    location = os.getenv('HOME')

    def __init__(self, parent=None):
        super().__init__(parent)
        self.changed = False
        self.allclasses = AllClasses()
        self.allteachers = AllTeachers()
        if os.path.exists(settings.save_dir()):
            self.allclasses.read_data(settings.save_dir())
            self.allteachers.read_data(settings.save_dir())

        title = 'Deputat - Übersicht'
        icon = os.path.join(self.icon_path, 'deputat.svg')
        self.setWindowTitle(title)
        self.setWindowIcon(QIcon(icon))
        self.setGeometry(200, 200, 1250, 500)

        self._build_menu()
        self.statusBar().showMessage('')

        self.main_widget = MainWidget(self)
        self.setCentralWidget(self.main_widget)


    def _build_menu(self):
        menu_bar = self.menuBar()
        file = menu_bar.addMenu("Datei")

        save = QAction(QIcon(os.path.join(self.icon_path, 'save.svg')), "Speichern", self)
        file.addAction(save)
        save.triggered.connect(self._save)

        load = file.addMenu(QIcon(os.path.join(self.icon_path, 'add.svg')), "Lade Daten")
        import_full = QAction(QIcon(os.path.join(self.icon_path, 'import.svg')), 'Vollständiges Set', self)
        import_teacher = QAction(QIcon(os.path.join(self.icon_path, 'add_teacher.svg')), 'Lehrer', self)
        import_class = QAction(QIcon(os.path.join(self.icon_path, 'add_class.svg')), 'Klassen', self)

        import_full.triggered.connect(self._import_full)
        import_teacher.triggered.connect(self._import_teacher)
        import_class.triggered.connect(self._import_class)

        load.addAction(import_full)
        load.addAction(import_teacher)
        load.addAction(import_class)

        export = file.addMenu(QIcon(os.path.join(self.icon_path, 'export.svg')), 'Exportieren')
        export_xlsx = QAction(QIcon(os.path.join(self.icon_path, 'excel.svg')), 'Exportieren als .xlsx', self)
        export_csv = export.addMenu(QIcon(os.path.join(self.icon_path, 'csv.svg')), 'Exportieren als .csv')
        export_csv_teacher = QAction(QIcon(os.path.join(self.icon_path, 'add_teacher.svg')), 'Lehrer exportieren', self)
        export_csv_classes = QAction(QIcon(os.path.join(self.icon_path, 'add_teacher.svg')), 'Klassen exportieren', self)
        export_xlsx.triggered.connect(self._export_xlsx)
        export_csv_teacher.triggered.connect(self._export_teachers)
        export_csv_classes.triggered.connect(self._export_classes)

        export_csv.addAction(export_csv_teacher)
        export_csv.addAction(export_csv_classes)
        export.addAction(export_xlsx)
        file.addSeparator()

        file.addAction(QAction(QIcon(os.path.join(self.icon_path, 'exit.svg')), "Quit", self))

        info = menu_bar.addMenu("Info")
        info.addAction(QAction(QIcon(os.path.join(self.icon_path, 'about.svg')), "Über", self))
        info.addAction(QAction(QIcon(os.path.join(self.icon_path, 'github.svg')), "GitHub", self))
        info.addAction(QAction(QIcon(os.path.join(self.icon_path, 'pypi.svg')), "PyPi", self))

        help = menu_bar.addMenu("Hilfe")
        help.addAction(QAction(QIcon(os.path.join(self.icon_path, 'mail.svg')), "Mail", self))
        help.addAction(QAction(QIcon(os.path.join(self.icon_path, 'github.svg')), "GitHub", self))
        help.addAction(QAction(QIcon(os.path.join(self.icon_path, 'readme.svg')), "Readme", self))


    def closeEvent(self, event):
        if self.changed:
            close = QuitPopUp('exit').get()
            if close:
                event.accept()
            else:
                event.ignore()
        else:
            event.accept()


    def _import_full(self):
        path = QFileDialog.getExistingDirectory(self, 'Odner wählen')
        if not self.location:
            self.location = path
        self.allclasses.read_data(path)
        self.allteachers.read_data(path)
        self.main_widget._refresh()


    def _import_teacher(self):
        path = QFileDialog.getOpenFileName(self, 'Lehrer wählen', self.location, "td-files (*.td)")[0]
        self.location = path
        self.allteachers.read_data(path, file=True)
        self.main_widget._refresh(self.main_widget)
        self.statusBar().showMessage(f'{os.path.split(path)[1]} importiert')


    def _import_class(self):
        path = QFileDialog().getOpenFileName(self, 'Klasse wählen', self.location, "cd-files (*.cd)")[0]
        self.location = path
        self.allclasses.read_data(path, file=True)
        self.main_widget._refresh(self.main_widget)
        self.statusBar().showMessage(f'{os.path.split(path)[1]} importiert')


    def _save(self):
        a = self.allclasses.save_data()
        b = self.allteachers.save_data()
        self.changed = False
        if a and b:
            self.statusBar().showMessage('erfolgreich gespeichert')
        else:
            self.statusBar().showMessage('speichern nicht erfolgreich!')


    def _export_classes(self):
        path = QFileDialog.getSaveFileName(self, 'File wählen', self.location, "csv-files (*.csv)")[0]
        to_save = pretty_out_classes(self.allclasses)
        to_save.to_csv(path, sep=',', index=False)
        self.statusBar().showMessage(f'{os.path.split(path)[1]} erfolgreich exportiert')


    def _export_teachers(self):
        path = QFileDialog.getSaveFileName(self, 'File wählen', self.location, "csv-files (*.csv)")[0]
        to_save = pretty_out_teachers(self.allclasses, self.allteachers)
        to_save.to_csv(path, sep=',', index=False)
        self.statusBar().showMessage(f'{os.path.split(path)[1]} erfolgreich exportiert')


    def _export_xlsx(self):
        import pandas as pd
        path = QFileDialog.getSaveFileName(self, 'File wählen', self.location, "excel-files (*.xlsx)")[0]
        lehrer = pretty_out_teachers(self.allclasses, self.allteachers)
        klasse = pretty_out_classes(self.allclasses)
        writer = pd.ExcelWriter(path)
        lehrer.to_excel(writer, 'Lehrer', index=False)
        klasse.to_excel(writer, 'Klassen', index=False)
        writer.save()
        self.statusBar().showMessage(f'{os.path.split(path)[1]} erfolgreich exportiert')



class MainWidget(QWidget):
    icon_path = settings.icon_dir()

    def __init__(self, parent=None):
        super().__init__(parent)
        self.mainwindow = parent

        self.last_tab = 0
        self.last_name = None # for showing teacher in searchbar after adding teacher to class

        # ----------------create GroupBoxes-------------
        self.createTLGB()
        self.createBLGB()
        self.createTRGB()
        self.createBRGB()
        # ----------------end GroupBoxes----------------

        # ----------------main Layout-------------------
        main_layout = QHBoxLayout()

        left_area = QVBoxLayout()
        right_area = QVBoxLayout()
        left_area.addWidget(self.TLGB)
        left_area.addWidget(self.BLGB)
        right_area.addWidget(self.TRGB)
        right_area.addWidget(self.BRGB)

        main_layout.addLayout(left_area)
        main_layout.addLayout(right_area)

        self.setLayout(main_layout)
        # ----------------end main Layout----------------


    def createTLGB(self):
        """
        places GroupBox to top-left
        """

        self.TLGB = QGroupBox('Suchoptionen für Klassen')
        layout = QVBoxLayout()
        sublayout_top = QHBoxLayout()
        sublayout_bot = QHBoxLayout()

        level_label = QLabel('Klassenstufe: ')
        self.select_level = QComboBox()
        self.fill_combo_level()

        self.hours_missing = QCheckBox('Nur Klassen mit fehlenden Stunden')
        self.done_classes = QCheckBox('Nur volltändige Klassen')
        self.spec_teacher = QCheckBox('Nur spezifische Lehrkraft')

        self.select_teacher = QComboBox()
        self.fill_combo_teacher()
        self.select_teacher.setDisabled(True)

        self.spec_teacher.toggled.connect(self.select_teacher.setEnabled)

        reset_button = QPushButton('Zurücksetzen')

        self.select_level.currentTextChanged.connect(self._search_classes)
        self.hours_missing.stateChanged.connect(self._search_classes)
        self.done_classes.stateChanged.connect(self._search_classes)
        self.spec_teacher.stateChanged.connect(self._search_classes)
        self.select_teacher.currentTextChanged.connect(self._search_classes)
        reset_button.clicked.connect(self._reset)

        sublayout_top.addWidget(level_label)
        sublayout_top.addWidget(self.select_level)
        sublayout_bot.addWidget(reset_button)
        #sublayout_bot.addWidget(search_button)

        layout.addLayout(sublayout_top)
        layout.addWidget(self.hours_missing)
        layout.addWidget(self.done_classes)
        layout.addWidget(self.spec_teacher)
        layout.addWidget(self.select_teacher)
        layout.addLayout(sublayout_bot)

        self.TLGB.setLayout(layout)

    def fill_combo_teacher(self):
        self.select_teacher.clear()
        self.select_teacher.addItems(['Lehrkraft auswählen'] + [t.name for t in self.mainwindow.allteachers.teachers])


    def fill_combo_level(self):
        self.select_level.clear()
        self.select_level.addItems(['Alle'] + self.mainwindow.allclasses.list_levels())


    def createBLGB(self):
        """
        places GroupBox to bottom-left
        """

        self.BLGB = QGroupBox('Reststunden Lehrer')
        self.blgb_layout = QHBoxLayout()
        self.list_area = QListWidget()
        self._build_teacher_list()
        self.blgb_layout.addWidget(self.list_area)
        self.BLGB.setLayout(self.blgb_layout)


    def createTRGB(self):
        """
        places GroupBox to top-left
        """

        self.TRGB = QGroupBox('Deputat')
        gb_layout = QVBoxLayout()
        self.tabs = QTabWidget()
        self._list_tabs(self.mainwindow.allclasses.classes)
        layout_tab_bot = QHBoxLayout()

        add_teacher_button = QPushButton('Lehrer Hinzufügen')
        add_teacher_button.setIcon(QIcon(os.path.join(self.icon_path, 'add.svg')))
        add_class_button = QPushButton('Klasse Hinzufügen')
        add_class_button.setIcon(QIcon(os.path.join(self.icon_path, 'add.svg')))

        add_teacher_button.clicked.connect(self._add_teacher)
        add_class_button.clicked.connect(self._add_class)

        layout_tab_bot.addWidget(add_teacher_button)
        layout_tab_bot.addWidget(add_class_button)
        gb_layout.addWidget(self.tabs)
        gb_layout.addLayout(layout_tab_bot)
        self.TRGB.setLayout(gb_layout)


    def createBRGB(self):
        """
        places GroupBox to bottom-right
        """

        self.BRGB = QGroupBox('Lehrer')

        layout = QVBoxLayout()
        top = QHBoxLayout()
        self.searchbar = QComboBox()
        self.fill_combo_teacher_search()
        top.addWidget(self.searchbar)

        self.selected = self._selected_teacher()

        layout.addLayout(top)
        layout.addLayout(self.selected)
        self.searchbar.currentTextChanged.connect(self._changed_search)
        self.BRGB.setLayout(layout)
        # ----------------end Layout----------------


    def fill_combo_teacher_search(self):
        self.searchbar.clear()
        self.searchbar.addItems(['Lehrkraft auswählen'] + [t.name for t in self.mainwindow.allteachers.teachers])


    def _build_teacher_list(self):
        self.list_area.clear()
        self.list_area.addItems(self.mainwindow.allteachers._list_teacher_hours())


    def _add_tab(self, obj):
        tab = QWidget()
        layout_tab = QVBoxLayout()
        for i in obj.subjects:
            item = self._class_info_item(i, obj.subjects[i], obj)
            layout_tab.addLayout(item)
        tab.setLayout(layout_tab)
        return tab


    def _selected_teacher(self):
        selected = QHBoxLayout()
        self.info = QLabel('')
        selected.addWidget(self.info)
        return selected


    def _changed_search(self):
        name = self.searchbar.currentText()
        text = ''
        for t in self.mainwindow.allteachers.teachers:
            if t.name == name:
                text = str(t)
        if name == 'Lehrkraft auswählen':
            self.info.setText('')
        else:
            self.info.setText(text)
        if self.last_name:
            for t in self.mainwindow.allteachers.teachers:
                if t.name == self.last_name:
                    self.searchbar.setCurrentText(self.last_name)
                    break


    def _list_available_teachers(self, subject):
        return ['Lehrkraft wählen'] + [f'{t.name} ({t.short}) - {t._get_hours_left()} Stunden übrig'
                                       for t in self.mainwindow.allteachers.teachers
                                       if subject in t.subjects]


    def _reset(self):
        self.hours_missing.setChecked(False)
        self.done_classes.setChecked(False)
        self.spec_teacher.setChecked(False)
        self.select_level.setCurrentText('Alle')
        self.select_teacher.setCurrentText('Lehrkraft auswählen')
        self._search_classes()


    def _search_classes(self):
        levels = self.select_level.currentText()
        hours_missing = self.hours_missing.checkState()
        done_classes = self.done_classes.checkState()
        teacher = None
        classes = []
        current_text = self.select_teacher.currentText()
        if self.spec_teacher.checkState() and current_text != 'Lehrkraft auswählen':
            for t in self.mainwindow.allteachers.teachers:
                if t.name == current_text:
                    teacher = t
                    break
        for c in self.mainwindow.allclasses.classes:
            if str(c.level) == levels or levels == 'Alle':
                classes.append(c)
        if hours_missing:
            for c in classes.copy():
                if not c.hours_missing(): #wenn keine stunden fehlen -> entferne Klasse
                    classes.remove(c)
        if done_classes:
            for c in classes.copy():
                if c.hours_missing(): #wenn Stunden fehlen -> entferne Klasse
                    classes.remove(c)
        if teacher:
            for c in classes.copy():
                if teacher.short not in c.list_teachers():
                    classes.remove(c)
        self._list_tabs(classes)


    def _list_tabs(self, classes):
        if self.tabs.currentIndex() != -1:
            self.last_tab = self.tabs.currentIndex()
        for i in list(range(self.tabs.count()))[::-1]:
            self.tabs.removeTab(i)
        liste = sorted([obj for obj in classes])
        for obj in liste:
            self.tabs.addTab(self._add_tab(obj), obj.get_fullname())
        self.tabs.setCurrentIndex(self.last_tab)


    def _class_info_item(self, short: str, info: list, obj):
        layout = QHBoxLayout()
        name = SUBJECT_SHORT_DICT[short]
        hours = str(info[0])
        teacher = info[1]
        name_label = QLabel(name)
        hours_label = QLabel(hours)
        teacher_selection = QComboBox()
        available_teachers = self._list_available_teachers(short)
        teacher_selection.addItems(available_teachers)
        if obj.subjects[short][1] == teacher:
            for i in available_teachers:
                if teacher in i:
                    teacher_selection.setCurrentText(i)
        teacher_selection.activated.connect(lambda: self.teacher_added(obj, teacher_selection.currentText(), short))
        layout.addWidget(name_label)
        layout.addWidget(hours_label)
        layout.addWidget(teacher_selection)
        return layout


    def teacher_added(self, klasse, text, short):
        teacher = None
        name = None
        for t in self.mainwindow.allteachers.teachers:
            if t.short in text:
                teacher = t
                break
        if not self.mainwindow.allclasses.backup:
            self.mainwindow.allclasses.backup = self.mainwindow.allclasses.classes.copy()
        for c in self.mainwindow.allclasses.classes:
            if c == klasse:
                for s in c.subjects:
                    if s != short:
                        continue
                    try:
                        c.subjects[s][1] = teacher.short
                        name = teacher.name
                    except AttributeError:
                        c.subjects[s][1] = 'null'
        self.last_name = name
        self._refresh()
        self.mainwindow.changed = True

    def _add_class(self):
        add_c = AddClassPopUp('Klasse Hinzufügen', self)
        add_c.setGeometry(100, 200, 500, 300)
        add_c.show()


    def _add_teacher(self):
        add_t = AddTeacherPopUp('Lehrer Hinzufügen', self)
        add_t.setGeometry(100, 200, 500, 300)
        add_t.show()


    def _refresh(self):
        self._changed_search()
        self._search_classes()
        self._build_teacher_list()
        self.fill_combo_teacher()
        self.fill_combo_level()
        self.fill_combo_teacher_search()


def run():
    import sys
    app = QApplication(sys.argv)
    gallery = MainWindow()
    gallery.show()
    sys.exit(app.exec_())



import os
import sys
import distro

def base_dir():
    return os.getcwd()

def icon_dir():
    return os.path.join(base_dir(), 'GUI', 'pictures')

def get_os():
    system = sys.platform
    if system.startswith('linux'):
        return ('Linux',) + distro.linux_distribution()
    if system.startswith('windows'):
        return ('Windows',)
    if system.startswith('darwin'):
        return ('MacOS',)

def save_dir():
    home = os.getenv('HOME')
    data = os.path.join(home, 'deputat_data', 'data')
    if not os.path.exists(data):
        os.mkdir(os.path.join(home, 'deputat_data'))
        os.mkdir(data)
    return data



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='deputat',
    version='1.0.3',
    description='deputat overview',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lfreist/deputat",
    author='lfreist',
    author_email='freist.leon@gmx.de',
    packages=find_packages(),
    package_data={'deputat': ['GUI/pictures/*.svg']},
    install_requires=[
              'PyQt5',
              'distro',
              'openpyxl',
              'pandas'
              ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        ],
)



import os
import sys
import pandas as pd

sys.path.append('../')

import settings


# dictionary for subjects
SUBJECT_LONG_DICT = {
    'Mathe': 'M',
    'Deutsch': 'D',
    'Englisch': 'E',
    'Sport': 'S',
    'Französisch': 'F',
    'Technik': 'T'
}

# reversed dictionary for subjects
SUBJECT_SHORT_DICT = {short: long for long, short in SUBJECT_LONG_DICT.items()}

save = settings.save_dir()


class Class:
    def __init__(self, level: int, name: str, subjects: dict):
        self.level = int(level)
        self.name = name
        self.subjects = subjects


    def __str__(self):
        return f'Klasse:\t {self.level}{self.name}\n' \
               f'Fächer:\t {self._get_subjects()}'

    def __gt__(self, other):
        if self.level == other.level:
            return self.name > other.name
        return self.level > other.level

    def __lt__(self, other):
        if self.level == other.level:
            return self.name < other.name
        return self.level < other.level

    def __eq__(self, other):
        return self.name == other.name and self.level == other.level

    def __repr__(self):
        return f'Class({self.level}, {self.name}, {self.subjects})'

    def list_it(self):
        return [self.level, self.name, ';'.join(self.subjects_to_list())]


    def subjects_to_list(self):
        liste = []
        for i in self.subjects:
            liste.append(str(i + f'-{self.subjects[i][0]}-{self.subjects[i][1]}'))
        return liste


    def _get_subjects(self):
        text = ''
        for i in self.subjects:
            text += i + f' ({self.subjects[i][0]} Stunden) bei {self.subjects[i][1]}\n\t\t '
        return text


    def get_fullname(self):
        return f'{self.level}{self.name}'


    def hours_missing(self):
        for i in self.subjects:
            if self.subjects[i][1] == 'null':
                return True
        return False


    def list_teachers(self):
        teachers = []
        for i in self.subjects:
            if self.subjects[i][1] not in teachers and self.subjects[i][1] != 'null':
                teachers.append(self.subjects[i][1])
        return teachers


class Teacher:
    def __init__(self, name: str, short: str, hours: int, subjects: list):
        assert hours >= 0, f'{name} hat weniger als 0 Stunden ({hours}).'
        self.name = name
        self.short = short
        self.hours = hours
        self.subjects = subjects
        self.hours_left = hours


    def __str__(self):
        subs = {}
        for s in self.subjects:
            s = SUBJECT_SHORT_DICT[s]
            subs[s] = 0
        for c in AllClasses.classes:
            for s in c.subjects:
                if SUBJECT_SHORT_DICT[s] in subs:
                    if c.subjects[s][1] == self.short:
                        subs[SUBJECT_SHORT_DICT[s]] += int(c.subjects[s][0])
        text = f'Name:\t {self.name} ({self.short})\n' \
               f'Fächer:\t {_build_string_from_dict(subs)}\n' \
               f'Stunden: {self.hours} ({self._get_hours_left()}h übrig)'
        return text


    def __repr__(self):
        return f'Teacher({self.name}, {self.short}, {self.hours}, {self.subjects})'


    def __eq__(self, other):
        return self.name == other.name and self.short == other.short


    def __lt__(self, other):
        return self.short < other.short


    def __gt__(self, other):
        return self.short > other.short


    def list_it(self):
        return [self.name, self.short, str(self.hours), ';'.join(self.subjects)]


    def _get_hours_left(self):
        teached_hours = []
        for c in AllClasses.classes:
            for s in c.subjects:
                if c.subjects[s][1] == self.short:
                    teached_hours.append(int(c.subjects[s][0]))
        return self.hours - sum(teached_hours)


class AllTeachers:
    teachers = []
    backup = []
    filename = 'lehrer.td'

    def read_data(self, path, file=None):
        if file:
            path, file = os.path.split(path)
            liste = read_data(path, 'td', file)
        else:
            liste = read_data(path, 'td')
        for i in liste:
            self.add_teacher(i)


    def save_data(self, location=save):
        save_data(self.filename, location)


    def _list_teacher_hours(self):
        list = []
        for t in self.teachers:
            subs = {}
            for s in t.subjects:
                subs[s] = 0
            for c in AllClasses.classes:
                for s in c.subjects:
                    if s in subs:
                        if c.subjects[s][1] == t.short:
                            subs[s] += int(c.subjects[s][0])
            list.append(f'{t.name}\t({t.short})\t-\t[{_build_string_from_dict(subs)}]')
        return list


    def add_teacher(self, teacher=None, name=None, short=None, hours=None, subjects=None):
        if not teacher:
            teacher = Teacher(name, short, hours, subjects)
        if teacher in self.teachers:
            return False
        self.teachers.append(teacher)
        return True


    def get_all_teachers(self) -> list:
        liste = []
        for t in self.teachers:
            if t.short not in liste:
                liste.append(t.short)
        return liste


class AllClasses:
    classes = []
    backup = []
    filename = 'klassen.cd'

    def read_data(self, path, file=None):
        if file:
            path, file = os.path.split(path)
            liste = read_data(path, 'cd', file)
        else:
            liste = read_data(path, 'cd')
        for i in liste:
            self.add_class(i)


    def save_data(self, location=save):
        save_data(self.filename, location)


    def list_levels(self):
        liste = []
        for c in self.classes:
            if str(c.level) not in liste:
                liste.append(str(c.level))
        return liste


    def add_class(self, klasse=None, level=None, name=None, subjects=None):
        if not klasse:
            klasse = Class(level, name, subjects)
        if klasse in self.classes:
            return False
        self.classes.append(klasse)
        return True


    def get_all_subjects(self) -> list:
        list = []
        for c in self.classes:
            for s in c.subjects:
                if s not in list:
                    list.append(s)
        return list


    def get_all_classes(self) -> list:
        liste = []
        for c in self.classes:
            if c.get_fullname() not in liste:
                liste.append(c.get_fullname())
        return liste


def _build_string_from_dict(subs: dict):
    text = ""
    for i in subs:
        text += f'{i}: {subs[i]}h, '
    return text[0:-2]


def read_data(path, typ, filename=None):
    liste = []
    if filename:
        if typ == 'cd':
            with open(os.path.join(path, filename), 'r') as file:
                lines = file.readlines()[1:]
            for line in lines:
                line = line.strip().split(',')
                line[-1] = line[-1].split(';')  # 'S-2-null'
                new = {}
                for fach in line[-1]:
                    fach = fach.split('-')  # ['S', '2', 'null']
                    fach[1] = int(fach[1])
                    new[fach[0]] = fach[1:]
                line[-1] = new
                liste.append(build_object(line, typ))
        elif typ == 'td':
            with open(os.path.join(path, filename), 'r') as file:
                lines = file.readlines()[1:]
            for line in lines:
                line = line.strip().split(',')
                line[-1] = line[-1].split(';')
                liste.append(build_object(line, typ))
    for f in os.listdir(path):
        if typ == 'cd':
            if f.endswith('.cd'):
                with open(os.path.join(path, f), 'r') as file:
                    lines = file.readlines()[1:]
                for line in lines:
                    line = line.strip().split(',')
                    line[-1] = line[-1].split(';')  # 'S-2-null'
                    new = {}
                    for fach in line[-1]:
                        fach = fach.split('-')  # ['S', '2', 'null']
                        fach[1] = int(fach[1])
                        new[fach[0]] = fach[1:]
                    line[-1] = new
                    liste.append(build_object(line, typ))
        elif typ == 'td':
            if f.endswith('.td'):
                with open(os.path.join(path, f), 'r') as file:
                    lines = file.readlines()[1:]
                for line in lines:
                    line = line.strip().split(',')
                    line[-1] = line[-1].split(';')
                    liste.append(build_object(line, typ))
    return liste


def build_object(line: list, typ: str) -> bool:
    if typ == 'td':
        name, short, hours, subjects = line[0], line[1], int(line[2]), line[3]
        return Teacher(name, short, hours, subjects)
    elif typ == 'cd':
        level, name, subjects = line[0], line[1], line[2]
        return Class(level, name, subjects)


def save_data(filename, path=save):
    path = os.path.join(path, filename)
    try:
        file = open(path, 'w')
        if filename.endswith('.td'):
            print('name,kürzel,stunden,fächer', file=file)
            for obj in AllTeachers.teachers:
                print(",".join(obj.list_it()), file=file)
        elif filename.endswith('.cd'):
            print('klassenstufe,name,fächer(fach-stundenzahl-lehrer)', file=file)
            for obj in AllClasses.classes:
                print(",".join([str(i) for i in obj.list_it()]), file=file)
        file.close()
    except TypeError as error:
        print(error)


def pretty_out_teachers(classes: AllClasses, teachers: AllTeachers):
    s_dict = {'Fächer': sorted(classes.get_all_subjects())}
    teachers_list = sorted(teachers.get_all_teachers())
    ret_df = pd.DataFrame(s_dict)
    t_dict = {}
    for t in sorted(teachers.teachers):
        infos = get_teachers_classes(classes, t)
        infos_list = sorted(infos)
        subs = [infos[i] for i in infos_list]
        subs = [f'{format_subs(i)}' for i in subs]
        t_dict[t.short] = subs
    for i in t_dict:
        ret_df[i] = t_dict[i]
    return ret_df


def format_subs(xs: list) -> str:
    hours = ''
    classes = ''
    if xs[0] == 0:
        return ''
    return f'{str(xs[0])} - {xs[1][:-2]}'


def pretty_out_classes(classes: AllClasses):
    all_subs = sorted(classes.get_all_subjects())
    s_dict = {'Fächer': all_subs}
    ret_df = pd.DataFrame(s_dict)
    c_dict = {}
    for c in sorted(classes.classes):
        info_dict = {}
        for s in all_subs:
            info_dict[s] = [0, 'null']
        for s in info_dict:
            if s in c.subjects:
                info_dict[s] = c.subjects[s]
        infos_list = sorted(info_dict)
        infos = [info_dict[i] for i in infos_list]
        infos = [f'{format_info(i)}' for i in infos]
        c_dict[c.get_fullname()] = infos
    for i in c_dict:
        ret_df[i] = c_dict[i]
    return ret_df


def format_info(xs: list) -> str:
    if xs[0] == 0:
        return ''
    return f'{xs[0]} - {xs[1]}'


def get_teachers_classes(classes: AllClasses, teacher: Teacher):
    ret_dict = {}
    for s in sorted(classes.get_all_subjects()):
        ret_dict[s] = [0, '']
    for c in sorted(classes.classes):
        for s in c.subjects:
            if c.subjects[s][1] == teacher.short:
                ret_dict[s][0] += c.subjects[s][0]
                ret_dict[s][1] += f'{c.get_fullname()}, '
    return ret_dict


if __name__ == '__main__':
    t = AllTeachers()
    t.read_data('/home/lfreist/Documents/projects/deputat/deputat/data')
    c = AllClasses()
    c.read_data('/home/lfreist/Documents/projects/deputat/deputat/data')


import sys

from PyQt5.QtWidgets import (QDialog, QDialogButtonBox, QFormLayout, QGroupBox, QLabel,
                             QLineEdit, QVBoxLayout, QMessageBox, QCheckBox, QSlider,
                             QSpinBox)

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QIntValidator

sys.path.append('../')

import settings
from script.deputat import AllTeachers, AllClasses, Class, SUBJECT_LONG_DICT


class AddTeacherPopUp(QDialog):
    icon_path = settings.icon_dir()
    NumGridRows = 3
    NumButtons = 4

    def __init__(self, name, parent=None):
        super().__init__(parent)
        self.mainwidget = parent
        self.create_form_GB()

        button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        button_box.accepted.connect(self.add)
        button_box.rejected.connect(self.reject)

        main_layout = QVBoxLayout()
        main_layout.addWidget(self.formGroupBox)
        main_layout.addWidget(button_box)
        self.setLayout(main_layout)

        self.setWindowTitle(name)

    def create_form_GB(self):
        self.formGroupBox = QGroupBox("Informationen")
        self.layout = QFormLayout()
        self.name, self.short = QLineEdit(), QLineEdit()
        self.subjects = [QCheckBox(s) for s in SUBJECT_LONG_DICT]
        self.layout.addRow(QLabel("Name:"), self.name)
        self.layout.addRow(QLabel("Kürzel:"), self.short)
        self.layout.addRow(QLabel("Fächer:"), self.subjects[0])
        for s in self.subjects[1:]:
            self.layout.addRow(QLabel(''), s)
        self.hours = QSlider(Qt.Horizontal)
        self.hours.setMinimum(1)
        self.hours.setMaximum(28)
        self.hours.setValue(22)
        self.hours.setTickPosition(QSlider.TicksAbove)
        self.hours.setTickInterval(1)
        self.hours.valueChanged.connect(self.update_label)
        self.hours_edit = QLineEdit('22')
        self.hours_edit.textChanged.connect(self.update_hours)
        self.layout.addRow(QLabel('Anzahl Stunden:'), QLabel(''))
        self.layout.addRow(self.hours_edit, self.hours)
        self.formGroupBox.setLayout(self.layout)


    def update_label(self):
        self.hours_edit.setText(str(self.hours.value()))


    def update_hours(self):
        try:
            self.hours.setValue(int(self.hours_edit.text()))
        except (TypeError, ValueError):
            pass

    def add(self):
        subs = []
        index = []
        for i, o in enumerate(self.subjects):
            if o.isChecked():
                index.append(i)
        for i, o in enumerate(SUBJECT_LONG_DICT):
            if i in index:
                subs.append(SUBJECT_LONG_DICT[o])
        if not subs or not self.name.text() or not self.short.text() or not self.hours.value():
            return
        self.mainwidget.mainwindow.allteachers.add_teacher(None, self.name.text(), self.short.text(), int(self.hours.value()), subs)
        self.mainwidget._refresh()
        self.mainwidget.mainwindow.statusBar().showMessage(f'Lehrer {self.name.text()} hinzugefügt')
        self.mainwidget.mainwindow.changed = True
        self.accept()


class AddClassPopUp(QDialog):
    icon_path = settings.icon_dir()
    NumGridRows = 3
    NumButtons = 4

    def __init__(self, name, parent=None):
        super().__init__(parent)
        self.mainwidget = parent
        
        self.valid = False
        
        self.create_form_GB()

        button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        button_box.accepted.connect(self.add)
        button_box.rejected.connect(self.reject)

        main_layout = QVBoxLayout()
        main_layout.addWidget(self.formGroupBox)
        main_layout.addWidget(button_box)
        self.setLayout(main_layout)

        self.setWindowTitle(name)

    def create_form_GB(self):
        self.formGroupBox = QGroupBox("Informationen")
        self.layout = QFormLayout()
        self.level, self.name = QLineEdit(), QLineEdit()
        self.level.setValidator(QIntValidator(1, 10))
        self.level.textChanged.connect(self.is_valid)
        self.name.textChanged.connect(self.is_valid)
        self.layout.addRow(QLabel("Jahrgangsstufe:"), self.level)
        self.layout.addRow(QLabel("Klassenname (a, b, etc.)"), self.name)
        self.layout.addRow(QLabel("Fächer"), QLabel("Stunden"))
        self.subjects = self.get_subjects()
        for l, s in self.subjects:
            self.layout.addRow(l, s)
        self.formGroupBox.setLayout(self.layout)


    def get_subjects(self):
        liste = []
        for s in SUBJECT_LONG_DICT:
            label = QLabel(f'    {s}')
            spin = QSpinBox()
            spin.setValue(0)
            spin.setMaximum(6)
            spin.setMinimum(0)
            liste.append((label, spin))
        return liste


    def is_valid(self):
        if not self.level.text() or not self.name.text():
            pass
        for c in self.mainwidget.mainwindow.allclasses.classes:
            if f'{c.level}{c.name}' == f'{self.level.text()}{self.name.text()}':
                self.level.setStyleSheet('Background: red')
                self.valid = False
                return
        self.level.setStyleSheet('Background: white')
        self.valid = True


    def add(self):
        subs = {}
        for i, o in enumerate(self.subjects):
            hours = int(o[1].text())
            name = o[0].text().strip()
            if hours:
                subs[SUBJECT_LONG_DICT[name]] = [hours, 'null']
        if not subs or not self.name.text() or not self.level.text() or not self.valid:
            return
        self.mainwidget.mainwindow.allclasses.add_class(None, int(self.level.text()), self.name.text(), subs)
        self.mainwidget._refresh()
        self.mainwidget.mainwindow.statusBar().showMessage(f'Klasse {self.level.text()}{self.name.text()} hinzugefügt')
        self.mainwidget.mainwindow.changed = True
        self.accept()



class QuitPopUp(QMessageBox):

    def __init__(self, name, parent=None):
        super().__init__(parent)
        self.setIcon(QMessageBox.Warning)
        self.setText("Ohne speichern verlassen?")
        self.setWindowTitle("Exit")
        self.setDetailedText("Diese Warnung kann fälschlicherweise angezeigt werden,"\
                             "wenn Änderungen manuell rückgängig gemacht wurden...")
        self.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        self.setDefaultButton(QMessageBox.No)

        self._close = None

        returnValue = self.exec()
        if returnValue == QMessageBox.Yes:
            self._close = True
        if returnValue == QMessageBox.No:
            self._close = False


    def get(self):
        return self._close